c$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
c H2+H2
c PCS
c May 7, 2007
c March 23, 2008 Modified to include |Delta j| >=4  transition
c              scaling
c PCS 4-19-2011 correction to fit file header read made
c$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
c subroutine to generated H2+H2 rotational quenching rate
c coefficients
c$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
       subroutine rateh2h2(vi,ji,vf,jf,vci,jci,vcf,jcf,temp,
     .   rate,qual,ic)
c$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
c Target Molecule:
c   vi initial vibrational quantum number
c   vf final vibrational quantum number
c   ji initial rotational quantum number
c   jf final rotational quantum number
c Collider Molecule:
c   vci initial vibrational quantum number
c   vcf final vibrational quantum number (assumed = vci)
c   jci initial rotational quantum number
c   jcf final rotational quantum number  (assumed = jci)
c   temp temperature to evaluate rate coefficient in K
c   rate value of rate coefficient to be returned in cm3/s
c   qual quality letter for accuracy of data (A is highest)
c   ic call index, 0 = initial call,  >0 all other calls
c   exo = 1 for exothermic process, exo=0 for endothermic
c
c molecule specific parameters (H2)
c   homo =1, homonuclear; homo=2, heteronuclear  
c   nv maximum number of vibrational states plus one
c   nj maximum number of rotational states plus one
c   ne total number of rovibrational states
c   nc total number of collider states (for H2 j=0, j=1)
c   ntrans total number of fitted transitions
c$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$     
       integer vi,ji,vf,jf,vci,jci,vcf,jcf,ic,ind,indx,ntrans,inc,
     .   ne,nv,nj,ve,je,vii,vff,ii,if,ini,inf,homo,initmp,exo,nc,
     .   inff, jff,j,i
       parameter(homo=1,ntrans=16,ne=301,nv=15,nj=32,nc=2)
       dimension indx(nv,nj)
       double precision temp,rate,en(nv,nj),a,b,c,d,e,f,g,h,
     .   fa(ne,ne,nc),fb(ne,ne,nc),fc(ne,ne,nc),fd(ne,ne,nc),
     .   fe(ne,ne,nc),ff(ne,ne,nc),deltat,
     .   fg(ne,ne,nc),fh(ne,ne,nc),error(ne,ne,nc),errot,kcm
c Boltzmann constant in cm^-1/K
       parameter(kcm=0.69503862d0)
       character quality*4,junk*38,elec*12,qual*4,junk1*76
       dimension quality(ne,ne,nc)
c
c test to see if call parameters out of bounds
c  adjusted for v=0 only
c  maximum v
       if((vi.gt.0).or.(vf.gt.0)) then
           write(6,*) 'vibrational state is too large'
           rate=-1.
           return
       end if
c   adjusted for jmax=8
c  maximum j 
c       if((ji.gt.(nj-1)).or.(jf.gt.(nj-1))) then
       if((ji.gt.8).or.(jf.gt.8)) then
           write(6,*) 'rotational state is too large'
           rate=-1.
           return
       end if
c  maximum/minimum temperature
       if((temp.gt.1.0d4).or.(temp.lt.1.0d-2)) then
           write(6,*) 'temperature out of range'
           rate=-1.
           return
       end if
c test to make sure no call parameters are negative
       if((vi.lt.0).or.(vf.lt.0).or.(ji.lt.0).or.(jf.lt.0)
     .   .or.(temp.lt.0.0d0)) then
           write(6,*) 'an input parameter is negative'
           rate=-1.
           return
       end if
c test for homonuclear case (i.e., only delta j even)
       if((homo.eq.1).and.(abs(mod((jf-ji)/1,2)).gt.0.5)) then
           write(6,*) 'delta j selection rule violated'
           rate=-1.
           return
       end if
c test for diatom-diatom case with no change in collider quantum numbers
       if((vci.ne.vcf).or.(jci.ne.jcf)) then
           write(6,*) 'change in perturber not available'
           rate=-1.
           return
       end if
c
c skip down if not first call
       if(ic.gt.0) goto 10
c zero all arrays
       do i=1,ne
          do j=1,ne
             do ii=1,nc
             fa(i,j,ii)=0.0d0
             fb(i,j,ii)=0.0d0
             fc(i,j,ii)=0.0d0
             fd(i,j,ii)=0.0d0
             fe(i,j,ii)=0.0d0
             ff(i,j,ii)=0.0d0
             fg(i,j,ii)=0.0d0
             fh(i,j,ii)=0.0d0
             end do
          end do
       end do
       do i=1,nv
          do j=1,nj
             en(i,j)=0.0d0
             indx(i,J)=0
          end do
       end do
c read in energy order for rovibrational levels
       open(unit=11,file="h2.energy.order")
       read(11,*) junk
       read(11,*) junk
       read(11,*) junk
       read(11,*) junk
       do i=1,ne
          read(11,99) ind,elec,ve,je,en(ve+1,je+1)
          indx(ve+1,je+1)=ind
       end do
       close(11)
c
c read in all fit parameters
       open(unit=12,file="h2h2r.fits")
c read header information and discard
       do j=1,17
           read(12,98) junk1
       end do
       do i=1,ntrans
          read(12,*) vii,vff,ii,if,qual,errot,a,b,c,d,e,f,g,h
          fa(vii,vff,ii)=a 
          fb(vii,vff,ii)=b 
          fc(vii,vff,ii)=c 
          fd(vii,vff,ii)=d 
          fe(vii,vff,ii)=e 
          ff(vii,vff,ii)=f 
          fg(vii,vff,ii)=g 
          fh(vii,vff,ii)=h
          quality(vii,vff,ii)=qual
          error(vii,vff,ii)=errot
       end do
       close(12)  
 10   continue
c
c determine index numbers from given quantum numbers
c        target
      ini=indx(vi+1,ji+1)
      inf=indx(vf+1,jf+1)
c        collider (here same species as target)
      inc=indx(vci+1,jci+1)
c
c test for excitation (endothermic) transitions and use
c   detailed balance to get rate coefficient from reverse process
      if(ini.lt.inf) then 
         exo=0
         initmp=ini
         ini=inf
         inf=initmp
      else
         exo=1
      end if
c
c determine rate coefficient
      a=fa(ini,inf,inc)
      b=fb(ini,inf,inc)
      c=fc(ini,inf,inc)
      d=fd(ini,inf,inc)
      e=fe(ini,inf,inc)
      f=ff(ini,inf,inc)
      g=fg(ini,inf,inc)
      h=fh(ini,inf,inc)
      qual=quality(ini,inf,inc)
      rate=10**(a+b/(d*temp/1.d3+1.d0) + c/(temp/1.d3+1.0d0)**2)
     .           + 10**(e+f/(g*temp/1.d3+1.d0)**h )
c
c #############################################################
c use scaling law for transitions not calculated
c  compute scaling law, but use last calculated value to
c  determine magnitude
      if(rate.eq.2.0) then
c scaling law for |delta v| = 0, |Delta j|>=4 
         if(abs(jf-ji).ge.4) then
         jff=ji-2
            if(exo.eq.0) then
               jff=jf-2
            end if
c
20         inff=indx(vf+1,jff+1)
c determine previous rate coefficient
           a=fa(ini,inff,inc)
           b=fb(ini,inff,inc)
           c=fc(ini,inff,inc)
           d=fd(ini,inff,inc)
           e=fe(ini,inff,inc)
           f=ff(ini,inff,inc)
           g=fg(ini,inff,inc)
           h=fh(ini,inff,inc)
           qual=quality(ini,inff,inc)
           rateu=10**(a+b/(d*temp/1.d3+1.d0) + c/(temp/1.d3+1.0d0)**2)
     .           + 10**(e+f/(g*temp/1.d3+1.d0)**h )
c empirical scaling, mostly based on Modified exponential energy gap
c law of Steinfield et al. 1991, J. Phys. Chem., 95, 9638, eq. 6,
c with A=2, delta=2/3, and beta/kT=1/300)
           deltat=(en(vi+1,jff+1)-en(vf+1,jf+1))/kcm
           rate=rateu*dexp(-deltat/300.0d0)
     .       /((2.0d0*jff+1.0d0)/(2.0d0*jf+1.0d0))
     .      *((1.d0+2.d0*deltat/temp)/(1.d0+3.d0*deltat/temp))**2

       end if
       end if   
c computes rate coefficient for excitation by detailed balance
c
      if(exo.eq.0) rate=rate*(2.0d0*jf+1.0d0)/(2.0d0*ji+1.0d0)
     .   *dexp((en(vi+1,ji+1)-en(vf+1,jf+1))/temp/kcm)
      return
 99   format(i5,A12,i5,i5,f10.2)
 98   format(A76)
      stop 
      end
